import React, { useEffect, useState } from "react";
import { ActivityIndicator, Dimensions, View } from "react-native";
import { NativeStackScreenProps } from "@react-navigation/native-stack";
import { FlatList } from "react-native";
import styled from "styled-components/native";
import Swiper from "react-native-swiper";
import Slide from "../components/Slide";
import HMedia from "../components/HMedia";
import VMedia from "../components/VMedia";
import { useQuery } from "react-query";
import { moviesAPI } from "../api";
const API_KEY = "1b2d2375eb7b1e0ef38c865b0d45ec64";

const Container = styled.FlatList``;

const Loader = styled.View`
  flex: 1;
  justify-content: center;
  align-items: center;
`;

const ListTitle = styled.Text`
  color: black;
  font-size: 18px;
  font-weight: 600;
  margin-left: 30px;
`;

const TrendingScroll = styled.FlatList`
  margin-top: 20px;
`;

const ListContainer = styled.View`
  margin-bottom: 40px;
`;

const CommingSoonTitle = styled(ListTitle)`
  margin-bottom: 20px;
`;

const VSeperator = styled.View`
  width: 20px;
`;

const HSeperator = styled.View`
  width: 20px;
`;

const { height: SCREEN_HEIGHT } = Dimensions.get("window");

const Movies: React.FC<NativeStackScreenProps<any, "Movies">> = () => {
  const movieKeyExtractor = (item) => item.id + "";
  const {
    isLoading: nowPlayingLoading,
    data: nowPlayingData,
    refetch: refetchNowLoading,
    isRefetching: isRefetchingNowLoading,
  } = useQuery("nowPlaying", moviesAPI.nowPlaying);
  const {
    isLoading: upCommingLoading,
    data: upCommingData,
    refetch: refetchUpcomming,
    isRefetching: isRefetchingUpcomming,
  } = useQuery("upComming", moviesAPI.upcomming);
  const {
    isLoading: trendingLoading,
    data: trendingData,
    refetch: refetchTrending,
    isRefetching: isRefetchingTrending,
  } = useQuery("trending", moviesAPI.treding);
  const loading = nowPlayingLoading || upCommingLoading || trendingLoading;
  const refreshing =
    isRefetchingNowLoading || isRefetchingUpcomming || isRefetchingTrending;

  const renderHMededia = ({ item }) => (
    <HMedia
      posterPath={item.poster_path}
      originalTitle={item.original_title}
      overview={item.overview}
      releaseDate={item.release_date}
    />
  );

  const renderVMedia = ({ item }) => (
    <VMedia
      posterPath={item.poster_path}
      originalTitle={item.original_title}
      voteAverage={item.vote_average}
    />
  );

  const onRefresh = async () => {
    // refetching manually
    refetchNowLoading();
    refetchUpcomming();
    refetchTrending();
    // notice to FlatList
  };
  console.log("is refreshing?: ", refreshing);
  return loading ? (
    <Loader>
      <ActivityIndicator size="large" />
    </Loader>
  ) : (
    <FlatList
      onRefresh={onRefresh}
      refreshing={refreshing}
      ListHeaderComponent={
        <>
          <Swiper
            horizontal
            loop
            autoplay={true}
            autoplayTimeout={3.5}
            showsButtons={false}
            showsPagination={false}
            containerStyle={{
              marginBottom: 30,
              width: "100%",
              height: SCREEN_HEIGHT / 4,
            }}
          >
            {nowPlayingData.results.map((movie) => (
              <Slide
                key={movie.id}
                backdropPath={movie.backdrop_path}
                posterPath={movie.poster_path}
                originalTitle={movie.original_title}
                voteAverage={movie.vote_average}
                overview={movie.overview}
              ></Slide>
            ))}
          </Swiper>
          <ListContainer>
            <ListTitle>Trending Movies</ListTitle>
            <TrendingScroll
              data={trendingData.results}
              renderItem={renderVMedia}
              keyExtractor={movieKeyExtractor}
              ItemSeparatorComponent={VSeperator}
              contentContainerStyle={{ paddingLeft: 30 }}
              horizontal
              showsHorizontalScrollIndicator={false}
            ></TrendingScroll>
          </ListContainer>
          <CommingSoonTitle>Comming Soon</CommingSoonTitle>
        </>
      }
      data={upCommingData.results}
      renderItem={renderHMededia}
      keyExtractor={movieKeyExtractor}
      ItemSeparatorComponent={HSeperator}
    ></FlatList>
  );
};
export default Movies;
